@@ -545,7 +545,7 @@ ctree_drag (GtkCTree *ctree, GtkCTreeNode *source_node,
                              GtkCTreeNode *new_parent,
                              GtkCTreeNode *new_sibling)
 {
-	ProjTreeWindow *ptw = gtk_object_get_data (GTK_OBJECT(ctree), "ptw");
+	ProjTreeWindow *ptw = g_object_get_data (G_OBJECT(ctree), "ptw");
 
 	if (!source_node) return FALSE;
 
@@ -1281,7 +1281,7 @@ ctree_new(void)
 	w = gtk_tree_view_new_with_model (GTK_TREE_MODEL(ptw->treestore));
 	ptw->treeview = GTK_TREE_VIEW (w);
 
-	gtk_object_set_data (GTK_OBJECT(w), "ptw", ptw);
+	g_object_set_data (G_OBJECT(w), "ptw", ptw);
 
 	/* Set up the columns in the treeview widget */
 	for (i=0; i<ptw->ncols; i++)
