@@ -22,6 +22,8 @@
 #include <config.h>
 #include <gnome.h>
 
+#include <gnc-date.h>
+
 #include "app.h"
 #include "ctree.h"
 #include "cur-proj.h"
@@ -41,8 +43,8 @@ typedef enum {
 	TIME_MONTH_COL,
 	TIME_WEEK_COL,
 	TIME_LASTWEEK_COL,
-	TIME_TODAY_COL,
 	TIME_YESTERDAY_COL,
+	TIME_TODAY_COL,
 	TIME_CURRENT_COL,
 	TITLE_COL,
 	DESC_COL,
@@ -213,6 +215,14 @@ ctree_stop_timer (GttProject *prj)
 
 /* ============================================================== */
 
+gboolean 
+ctree_has_focus (ProjTreeWindow *ptw)
+{
+	return GTK_WIDGET_HAS_FOCUS (GTK_WIDGET(ptw->ctree));
+}
+
+/* ============================================================== */
+
 static GtkCTreeNode *
 get_focus_row (GtkCTree *ctree)
 {
@@ -304,10 +314,22 @@ set_focus_project (ProjTreeWindow *ptw, GttProject *prj)
 static int
 widget_key_event(GtkCTree *ctree, GdkEvent *event, gpointer data)
 {
+	static int return_key_pressed = FALSE;
 	ProjTreeNode *ptn;
 	GtkCTreeNode *rownode;
 	GdkEventKey *kev = (GdkEventKey *)event;
 
+	if ((GDK_KEY_PRESS == event->type)  &&
+	     gtk_widget_is_focus (GTK_WIDGET(ctree)) &&
+	    (kev->keyval == GDK_Return)) 
+	{ 
+		/* Avoid toggling timer when receiving a KEY_RELEASE
+		 * if the KEY_PRESS was not received previously. 
+		 * (because KEY_PRESS went to another window.) */
+		return_key_pressed = TRUE;
+		return TRUE;
+	}
+
 	if (GDK_KEY_RELEASE != event->type) return FALSE;
 	if (FALSE == gtk_widget_is_focus (GTK_WIDGET(ctree))) return FALSE;
 	
@@ -315,10 +337,11 @@ widget_key_event(GtkCTree *ctree, GdkEvent *event, gpointer data)
 	{
 		case GDK_Return:
 			rownode = get_focus_row(ctree);
-			if (rownode)
+			if (rownode && return_key_pressed)
 			{
 				ptn = gtk_ctree_node_get_row_data(ctree, rownode);
 				toggle_timer_for_row (ptn->ptw, ptn);
+				return_key_pressed = FALSE;
 			}
 			return TRUE;
 		case GDK_Up:
@@ -515,12 +538,12 @@ click_column(GtkCList *clist, gint col, gpointer data)
 		case TIME_CURRENT_COL:
 			prlist = project_list_sort_current (prlist);
 			break;
-		case TIME_TODAY_COL:
-			prlist = project_list_sort_day (prlist);
-			break;
 		case TIME_YESTERDAY_COL:
 			prlist = project_list_sort_yesterday (prlist);
 			break;
+		case TIME_TODAY_COL:
+			prlist = project_list_sort_day (prlist);
+			break;
 		case TIME_WEEK_COL:
 			prlist = project_list_sort_week (prlist);
 			break;
@@ -654,7 +677,7 @@ ctree_drag (GtkCTree *ctree, GtkCTreeNode *source_node,
                              GtkCTreeNode *new_parent,
                              GtkCTreeNode *new_sibling)
 {
-	ProjTreeWindow *ptw = gtk_object_get_data (GTK_OBJECT(ctree), "ptw");
+	ProjTreeWindow *ptw = g_object_get_data (G_OBJECT(ctree), "ptw");
 
 	if (!source_node) return FALSE;
 
@@ -740,8 +763,8 @@ ctree_init_cols (ProjTreeWindow *ptw)
 	i++; ptw->cols[i] = TIME_MONTH_COL;
 	i++; ptw->cols[i] = TIME_WEEK_COL;
 	i++; ptw->cols[i] = TIME_LASTWEEK_COL;
-	i++; ptw->cols[i] = TIME_TODAY_COL;
 	i++; ptw->cols[i] = TIME_YESTERDAY_COL;
+	i++; ptw->cols[i] = TIME_TODAY_COL;
 	i++; ptw->cols[i] = TIME_CURRENT_COL;
 	i++; ptw->cols[i] = TITLE_COL;
 	i++; ptw->cols[i] = DESC_COL;
@@ -773,13 +796,6 @@ ctree_init_cols (ProjTreeWindow *ptw)
 					_("Time spent under the current diary entry.");
 				ptw->col_width_set[i] = FALSE;
 				break;
-			case TIME_TODAY_COL:
-				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
-				ptw->col_titles[i] = _("Today");
-				ptw->col_tooltips[i] =  
-					_("Time spent on this project today.");
-				ptw->col_width_set[i] = FALSE;
-				break;
 			case TIME_YESTERDAY_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
 				ptw->col_titles[i] = _("Yesterday");
@@ -787,6 +803,13 @@ ctree_init_cols (ProjTreeWindow *ptw)
 					_("Time spent on this project yesterday.");
 				ptw->col_width_set[i] = FALSE;
 				break;
+			case TIME_TODAY_COL:
+				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
+				ptw->col_titles[i] = _("Today");
+				ptw->col_tooltips[i] =  
+					_("Time spent on this project today.");
+				ptw->col_width_set[i] = FALSE;
+				break;
 			case TIME_WEEK_COL:
 				ptw->col_justify[i] = GTK_JUSTIFY_CENTER;
 				ptw->col_titles[i] = _("Week");
@@ -1008,16 +1031,16 @@ ctree_update_column_visibility (ProjTreeWindow *ptw)
 			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
 				i, config_show_title_current);
 			break;
-		case TIME_TODAY_COL:
-			default_col_width (ptw, i, "-00:00:00");
-			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
-				 i, config_show_title_day);
-			break;
 		case TIME_YESTERDAY_COL:
 			default_col_width (ptw, i, "-00:00:00");
 			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
 				 i, config_show_title_yesterday);
 			break;
+		case TIME_TODAY_COL:
+			default_col_width (ptw, i, "-00:00:00");
+			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
+				 i, config_show_title_day);
+			break;
 		case TIME_WEEK_COL:
 			default_col_width (ptw, i, "-00:00:00");
 			gtk_clist_set_column_visibility (GTK_CLIST(ptw->ctree),
@@ -1097,23 +1120,23 @@ ctree_update_column_visibility (ProjTreeWindow *ptw)
 
 /* ============================================================== */
 
-#define PRT_TIME(SLOT) {						\
-	ptn->col_values[i] =  ptn->SLOT##_timestr;			\
-	if (config_show_title_##SLOT) {					\
-		time_t secs;						\
-		if (expand) {						\
-			secs = gtt_project_get_secs_##SLOT(prj);	\
-		} else {						\
-			secs = gtt_project_total_secs_##SLOT(prj); 	\
-		}							\
-		if (0 < secs || ((0 == secs) && (prj == cur_proj))) { 	\
-			print_hours_elapsed (ptn->SLOT##_timestr, 24,	\
-				secs, config_show_secs);		\
-		} else {						\
-			ptn->SLOT##_timestr[0] = '-';			\
-			ptn->SLOT##_timestr[1] = 0x0;			\
-		}							\
-	}								\
+#define PRT_TIME(SLOT) {                                        \
+   ptn->col_values[i] =  ptn->SLOT##_timestr;                   \
+   if (config_show_title_##SLOT) {                              \
+      time_t secs;                                              \
+      if (expand) {                                             \
+         secs = gtt_project_get_secs_##SLOT(prj);               \
+      } else {                                                  \
+         secs = gtt_project_total_secs_##SLOT(prj);             \
+      }                                                         \
+      if (0 < secs || ((0 == secs) && (prj == cur_proj))) {     \
+         qof_print_hours_elapsed_buff (ptn->SLOT##_timestr, 24, \
+                 secs, config_show_secs);                       \
+      } else {                                                  \
+         ptn->SLOT##_timestr[0] = '-';                          \
+         ptn->SLOT##_timestr[1] = 0x0;                          \
+      }                                                         \
+   }                                                            \
 }
 
 static void 
@@ -1134,12 +1157,12 @@ stringify_col_values (ProjTreeNode *ptn, gboolean expand)
 		case TIME_CURRENT_COL:
 			PRT_TIME(current);
 			break;
-		case TIME_TODAY_COL:
-			PRT_TIME(day);
-			break;
 		case TIME_YESTERDAY_COL:
 			PRT_TIME(yesterday);
 			break;
+		case TIME_TODAY_COL:
+			PRT_TIME(day);
+			break;
 		case TIME_WEEK_COL:
 			PRT_TIME(week);
 			break;
@@ -1183,7 +1206,7 @@ stringify_col_values (ProjTreeNode *ptn, gboolean expand)
 				time_t secs;
 				secs = gtt_project_get_estimated_start(prj);
 				if (0 < secs) {
-					print_date (ptn->start_timestr, 24, secs);
+					qof_print_date_buff (ptn->start_timestr, 24, secs);
 				} else {
 					ptn->start_timestr[0] = '-';
 					ptn->start_timestr[1] = 0x0;
@@ -1196,7 +1219,7 @@ stringify_col_values (ProjTreeNode *ptn, gboolean expand)
 				time_t secs;
 				secs = gtt_project_get_estimated_end(prj);
 				if (0 < secs) {
-					print_date (ptn->end_timestr, 24, secs);
+					qof_print_date_buff (ptn->end_timestr, 24, secs);
 				} else {
 					ptn->end_timestr[0] = '-';
 					ptn->end_timestr[1] = 0x0;
@@ -1209,7 +1232,7 @@ stringify_col_values (ProjTreeNode *ptn, gboolean expand)
 				time_t secs;
 				secs = gtt_project_get_due_date(prj);
 				if (0 < secs) {
-					print_date (ptn->due_timestr, 24, secs);
+					qof_print_date_buff (ptn->due_timestr, 24, secs);
 				} else {
 					ptn->due_timestr[0] = '-';
 					ptn->due_timestr[1] = 0x0;
@@ -1325,12 +1348,10 @@ static void
 refresh_list (ProjTreeWindow *ptw, GList *prjlist)
 {
 	GList *node;
-	GtkCTree *tree_w = ptw->ctree;
 
 	/* now, draw each project */
 	for (node = prjlist; node; node = node->next) 
 	{
-		int i;
 		gboolean expand;
 		ProjTreeNode *ptn;
 		GttProject *prj = node->data;
@@ -1443,7 +1464,7 @@ ctree_new(void)
 	w = gtk_ctree_new_with_titles(ptw->ncols, i, ptw->col_titles);
 	ptw->ctree = GTK_CTREE(w);
 
-	gtk_object_set_data (GTK_OBJECT(w), "ptw", ptw);
+	g_object_set_data (G_OBJECT(w), "ptw", ptw);
 
 	for (i=0; i<ptw->ncols; i++)
 	{
@@ -1477,6 +1498,8 @@ ctree_new(void)
 			   G_CALLBACK(widget_button_event), ptw);
 	g_signal_connect(G_OBJECT(w), "key_release_event",
 			   G_CALLBACK(widget_key_event), ptw);
+	g_signal_connect(G_OBJECT(w), "key_press_event",
+			   G_CALLBACK(widget_key_event), ptw);
 	g_signal_connect(G_OBJECT(w), "tree_select_row",
 			   G_CALLBACK(tree_select_row), NULL);
 	g_signal_connect(G_OBJECT(w), "click_column",
@@ -1576,7 +1599,7 @@ ctree_setup (ProjTreeWindow *ptw, GList *prjlist)
 		for (node = prjlist; node; node = node->next) 
 		{
 			GttProject *prj = node->data;
-			ctree_add(ptw, prj, NULL);
+			ctree_add(ptw, prj);
 		}
 		gtk_clist_thaw(GTK_CLIST(tree_w));
 	} else {
@@ -1677,8 +1700,8 @@ ptn_destroy (ProjTreeNode *ptn)
 }
 #endif
 
-void
-ctree_add (ProjTreeWindow *ptw, GttProject *p, GtkCTreeNode *parent)
+static void
+do_ctree_add (ProjTreeWindow *ptw, GttProject *p, GtkCTreeNode *parent)
 {
 	ProjTreeNode *ptn;
 	GList *n;
@@ -1714,13 +1737,19 @@ ctree_add (ProjTreeWindow *ptw, GttProject *p, GtkCTreeNode *parent)
 	for (n=gtt_project_get_children(p); n; n=n->next)
 	{
 		GttProject *sub_prj = n->data;
-		ctree_add (ptw, sub_prj, ptn->ctnode);
+		do_ctree_add (ptw, sub_prj, ptn->ctnode);
 	}
 	
 	/* Set the focus row to the newly inserted project. */
 	set_focus_project (ptw, p);
 }
 
+void
+ctree_add (ProjTreeWindow *ptw, GttProject *p)
+{
+	do_ctree_add (ptw, p, NULL);
+}
+
 /* ============================================================== */
 
 void
@@ -1764,7 +1793,7 @@ ctree_insert_before (ProjTreeWindow *ptw, GttProject *p, GttProject *sibling)
 	for (n=gtt_project_get_children(p); n; n=n->next)
 	{
 		GttProject *sub_prj = n->data;
-		ctree_add (ptw, sub_prj, ptn->ctnode);
+		do_ctree_add (ptw, sub_prj, ptn->ctnode);
 	}
 	
 	/* Set the focus row to the newly inserted project. */
@@ -1821,7 +1850,7 @@ ctree_insert_after (ProjTreeWindow *ptw, GttProject *p, GttProject *sibling)
 	for (n=gtt_project_get_children(p); n; n=n->next)
 	{
 		GttProject *sub_prj = n->data;
-		ctree_add (ptw, sub_prj, ptn->ctnode);
+		do_ctree_add (ptw, sub_prj, ptn->ctnode);
 	}
 	
 	/* Set the focus row to the newly inserted project. */
@@ -1928,7 +1957,7 @@ ctree_get_expander_state (ProjTreeWindow *ptw)
 	ProjTreeNode *ptn;
 	gboolean expanded, is_leaf;
 
-	if (!ptw) return;
+	if (!ptw) return NULL;
 	ctree = ptw->ctree;
 
 	ptw->num_rows = gtt_project_list_total();
