@@ -23,20 +23,25 @@
 #include <gconf/gconf.h>
 #include <glade/glade.h>
 #include <gnome.h>
-#include <guile/gh.h>
 #include <libgnomeui/gnome-window-icon.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libguile.h>
 #include <signal.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <utime.h>
+#if defined(HAVE_DECL_WNOHANG) && defined(HAVE_WAITPID)
+#include <sys/wait.h>
+#endif
+
+#include <qof.h>
 
 #include "app.h"
 #include "ctree.h"
 #include "ctree-gnome2.h"
 #include "cur-proj.h"
-#include "dialog.h"
 #include "err-throw.h"
 #include "file-io.h"
 #include "gtt.h"
@@ -45,16 +50,13 @@
 #include "menucmd.h"
 #include "prefs.h"
 #include "timer.h"
+#include "toolbar.h"
 #include "xml-gtt.h"
 
-
 char *first_proj_title = NULL;  /* command line over-ride */
 
 static gboolean first_time_ever = FALSE;  /* has gtt ever run before? */
 
-/* SM == session management */
-#define USE_SM
-
 
 const char *
 gtt_gettext(const char *s)
@@ -141,7 +143,25 @@ void
 unlock_gtt(void)
 {
 	log_exit();
+	run_shell_command (cur_proj, FALSE);
 	unlink(build_lock_fname());
+
+	/* cleanup the guts. */
+#if 0
+/* Don't -- its currently buggy, recursive, will hang and whack data */
+	project_list_destroy ();
+#endif
+
+	/* Perform a clean shutdown of QOF subsystem. */
+	qof_query_shutdown ();
+	qof_object_shutdown ();
+	guid_shutdown ();
+	gnc_engine_string_cache_destroy ();
+
+	/* gnome shutdown */
+#if 0
+	gnome_vfs_shutdown ();
+#endif  
 }
 
 /* Return a 1 if the indicated directory did not exist, and
@@ -195,7 +215,7 @@ post_read_data(void)
 	toolbar_set_states();
 	init_timer();
 
-	/* plugins need to be added to the main menus dynamically,
+	/* Plugins need to be added to the main menus dynamically,
 	 * after the config file has been read */
 	menus_add_plugins (GNOME_APP(app_window));
 	log_start();
@@ -203,15 +223,17 @@ post_read_data(void)
 }
 
 static void 
-read_data_err_run_or_abort (GtkWidget *w, gint butnum)
+read_data_err_run_or_abort (GtkDialog *w, gint response_id)
 {
-	if (butnum == 1)
+	if ((GTK_RESPONSE_OK == response_id) ||
+	    (GTK_RESPONSE_YES == response_id))
 	{
-		gtk_main_quit();
+		gtk_widget_destroy (GTK_WIDGET(w));
+		post_read_data();
 	}
 	else
 	{
-		post_read_data();
+		gtk_main_quit();
 	}
 }
 
@@ -325,11 +347,16 @@ read_data(void)
 			_("Do you want to continue?"),
 			NULL);
 
-	msgbox_ok_cancel(_("Error"),
-			 qmsg,
-			 GTK_STOCK_YES, 
-			 GTK_STOCK_NO,
-			 G_CALLBACK(read_data_err_run_or_abort));
+	GtkWidget *mb;
+	mb = gtk_message_dialog_new (NULL,
+	         GTK_DIALOG_MODAL,
+	         GTK_MESSAGE_ERROR,
+	         GTK_BUTTONS_YES_NO,
+	         qmsg);
+	g_signal_connect (G_OBJECT(mb), "response",
+	         G_CALLBACK (read_data_err_run_or_abort), 
+	         NULL);
+	gtk_widget_show (mb);
 	g_free (qmsg); 
 	g_free (errmsg);
 }
@@ -341,16 +368,18 @@ post_read_config(void)
 }
 
 static void 
-read_config_err_run_or_abort (GtkWidget *w, gint butnum)
+read_config_err_run_or_abort (GtkDialog *w, gint response_id)
 {
-	if (butnum == 1)
+	if ((GTK_RESPONSE_OK == response_id) ||
+	    (GTK_RESPONSE_YES == response_id))
 	{
-		gtk_main_quit();
+		gtk_widget_destroy (GTK_WIDGET(w));
+		first_time_ever = TRUE;
+		post_read_config();
 	}
 	else
 	{
-		first_time_ever = TRUE;
-		post_read_config();
+		gtk_main_quit();
 	}
 }
 
@@ -375,11 +404,16 @@ read_config(void)
 			_("Shall I setup a new configuration?"),
 			NULL);
 
-		msgbox_ok_cancel(_("Error"),
-			 qmsg,
-			 GTK_STOCK_YES, 
-			 GTK_STOCK_NO,
-			 G_CALLBACK(read_config_err_run_or_abort));
+		GtkWidget *mb;
+		mb = gtk_message_dialog_new (NULL,
+		         GTK_DIALOG_MODAL,
+		         GTK_MESSAGE_ERROR,
+		         GTK_BUTTONS_YES_NO,
+		         qmsg);
+		g_signal_connect (G_OBJECT(mb), "response",
+		         G_CALLBACK (read_config_err_run_or_abort), 
+		         NULL);
+		gtk_widget_show (mb);
 		g_free (qmsg); 
 		g_free (errmsg);
 	}
@@ -428,7 +462,7 @@ make_backup (const char * filename)
 	struct stat old_stat;
 	struct utimbuf ub;
 	int suffix=0;
-	int lm, i;
+	int lm;
 	int rc;
 
 	/* Figure out how far to back up.  This computes a
@@ -543,10 +577,15 @@ save_properties (void)
 	{
 		char *errmsg = gtt_err_to_string (errcode, NULL);
 
-		msgbox_ok(_("Warning"),
-		     errmsg,
-		     GTK_STOCK_OK,
-		     NULL);
+		GtkWidget *mb;
+		mb = gtk_message_dialog_new (NULL,
+		         GTK_DIALOG_MODAL,
+		         GTK_MESSAGE_WARNING,
+		         GTK_BUTTONS_CLOSE,
+		         errmsg);
+		g_signal_connect (G_OBJECT(mb), "response",
+		         G_CALLBACK (gtk_widget_destroy), mb);
+		gtk_widget_show (mb);
 		g_free (errmsg);
 	}
 }
@@ -582,10 +621,16 @@ save_projects (void)
 	{
 		char *errmsg = gtt_err_to_string (errcode, xml_filepath);
 
-		msgbox_ok(_("Warning"),
-		     errmsg,
-		     GTK_STOCK_OK,
-		     NULL);
+		GtkWidget *mb;
+		mb = gtk_message_dialog_new (NULL,
+		         GTK_DIALOG_MODAL,
+		         GTK_MESSAGE_WARNING,
+		         GTK_BUTTONS_CLOSE,
+		         errmsg);
+		g_signal_connect (G_OBJECT(mb), "response",
+		         G_CALLBACK (gtk_widget_destroy), mb);
+		gtk_widget_show (mb);
+		g_free (errmsg);
 		g_free (errmsg);
 	}
 
@@ -594,8 +639,6 @@ save_projects (void)
 
 
 
-#ifdef USE_SM
-
 /*
  * session management
  */
@@ -647,8 +690,6 @@ session_die(GnomeClient *client)
 	app_quit(NULL, NULL);
 }
 
-#endif /* USE_SM */
-
 
 static void
 got_signal (int sig)
@@ -662,27 +703,28 @@ got_signal (int sig)
 
 
 static void 
-guile_inner_main(int argc, char **argv)
+guile_inner_main(void *closure, int argc, char **argv)
 {
 	gtk_main();
-
 	unlock_gtt();
-
-#if 0
-	/* Clean things up on exit.  But this is pretty pointless */
-	gtk_widget_destroy (app_window);
-	project_list_destroy ();
-#endif
 }
 
+#if defined (HAVE_DECL_WNOHANG) && defined (HAVE_WAITPID)
+inline RETSIGTYPE sigchld_handler(int unused) 
+{
+	while(waitpid(-1, NULL, WNOHANG) > 0) {}
+}
+#endif
 
 int 
 main(int argc, char *argv[])
 {
+#if defined (HAVE_DECL_WNOHANG) || defined (HAVE_DECL_SA_NOCLDWAIT)
+	struct sigaction reapchildren;
+	memset(&reapchildren, 0, sizeof reapchildren);
+#endif /*  WNOHANG/SA_NOCLDWAIT */
+
 	static char *geometry_string = NULL;
-#ifdef USE_SM
-	GnomeClient *client;
-#endif /* USE_SM */
 	static const struct poptOption geo_options[] =
 	{
 		{"geometry", 'g', POPT_ARG_STRING, &geometry_string, 0,
@@ -697,24 +739,38 @@ main(int argc, char *argv[])
 		                   GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-cromagnon.png");
 
+	gnome_vfs_init ();
+
 	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
-#ifdef USE_SM
-	client = gnome_master_client();
+	GnomeClient *client = gnome_master_client();
 	g_signal_connect(G_OBJECT(client), "save_yourself",
 			   G_CALLBACK(save_state), (gpointer) argv[0]);
 	g_signal_connect(G_OBJECT(client), "die",
 			   G_CALLBACK(session_die), NULL);
-#endif /* USE_SM */
 
 	glade_init();
 
 	/* gconf init is needed by gtkhtml */
 	gconf_init (argc, argv, NULL);
 
+#ifdef HAVE_DECL_WNOHANG
+	/* Create a signal handler to reap zombie processes.  Most portable */
+	reapchildren.sa_flags=SA_NOCLDSTOP;
+	reapchildren.sa_handler=sigchld_handler;
+	sigaction(SIGCHLD, &reapchildren, NULL);
+#elif defined (HAVE_DECL_SA_NOCLDWAIT)
+	/* Specify autoreaping using sigaction flag.  Next most portable */
+	memset(&reapchildren, 0, sizeof reapchildren);
+	reapchildren.sa_flags = SA_NOCLDWAIT;
+	sigaction(SIGCHLD, &reapchildren, NULL);
+#else
+	/* Old SysVr3 way of specifying auto reaping.  Inconsistently supported */
 	signal (SIGCHLD, SIG_IGN);
+#endif /* Signal handler to auto-reap zombies */
+
 	signal (SIGINT, got_signal);
 	signal (SIGTERM, got_signal);
 	lock_gtt();
@@ -723,6 +779,14 @@ main(int argc, char *argv[])
 	g_signal_connect(G_OBJECT(app_window), "delete_event",
 			   G_CALLBACK(app_quit), NULL);
 
+	/* Perform QOF-specific initialization. */
+	gnc_engine_get_string_cache();
+	guid_init();
+	qof_object_initialize ();
+	qof_query_init ();
+	qof_book_register ();
+	gtt_project_obj_register();
+
 #if DEVEL_VERSION_WARNING
 	msgbox_ok_cancel(_("Warning"),
 		"WARNING !!! Achtung !!! Avertisment !!!\n"
@@ -740,7 +804,7 @@ main(int argc, char *argv[])
 	read_config();
 #endif
 
-	gh_enter(argc, argv, guile_inner_main);
+	scm_boot_guile (argc, argv, guile_inner_main, NULL);
 	return 0; /* not reached !? */
 }
 
