@@ -17,7 +17,6 @@
  */
 
 #include "config.h"
-#include <errno.h>
 #include <glade/glade.h>
 #include <glib.h>
 #include <gnome.h>
@@ -99,7 +98,6 @@ edit_plugin_redraw_tree (struct PluginEditorDialog_s *ped)
 	for (i=0; i<ped->menus->len; i++)
 	{
 		if (GNOME_APP_UI_ENDOFINFO == uientry[i].type) break;
-
    	if (0 == rc)
 		{
 			gtk_tree_store_append (ped->treestore, &iter, NULL);
@@ -134,12 +132,17 @@ edit_plugin_widgets_to_item (PluginEditorDialog *dlg, GnomeUIInfo *gui)
 
 	if (!path)
 	{
-		msgbox_ok(_("Warning"),
-	   	          _("You must specify a complete filepath to the report, "
-	   	            "including a leading slash.  The file that you specify "
-	   	            "must exist."),
-	      	       GTK_STOCK_OK,
-	         	    NULL);
+		GtkWidget *mb;
+		mb = gtk_message_dialog_new (NULL,
+		         GTK_DIALOG_MODAL,
+		         GTK_MESSAGE_WARNING,
+		         GTK_BUTTONS_CLOSE,
+	   	      _("You must specify a complete filepath to the report, "
+	   	        "including a leading slash.  The file that you specify "
+	   	        "must exist."));
+		g_signal_connect (G_OBJECT(mb), "response",
+		         G_CALLBACK (gtk_widget_destroy), mb);
+		gtk_widget_show (mb);
 	}
 
 	tip = gtk_entry_get_text (dlg->plugin_tooltip);
@@ -201,7 +204,6 @@ edit_plugin_tree_selection_changed_cb (GtkTreeSelection *selection, gpointer dat
 	gboolean have_selection;
 
 	have_selection = gtk_tree_selection_get_selected (selection, &model, &iter);
-	
 	dlg->do_redraw = FALSE;
 	if (dlg->have_selection)
 	{
@@ -477,10 +479,7 @@ static void
 edit_plugin_delete_cb (GtkWidget * w, gpointer data)
 {
 	int row;
-	GnomeUIInfo *curr_item;
 	GnomeUIInfo *sysmenus;
-	GtkTreeModel *model;
-	GValue val = {G_TYPE_INVALID};
 	PluginEditorDialog *dlg = data;
 
 	if (FALSE == dlg->have_selection) return;
@@ -494,6 +493,7 @@ edit_plugin_delete_cb (GtkWidget * w, gpointer data)
 	if (GNOME_APP_UI_ENDOFINFO == sysmenus[row].type) return;
 
 	dlg->menus = g_array_remove_index (dlg->menus, row);
+	/* XXX mem leak .. should delete ui item */
 
 	/* Redraw the tree */
 	edit_plugin_redraw_tree (dlg);
@@ -503,6 +503,129 @@ edit_plugin_delete_cb (GtkWidget * w, gpointer data)
 	edit_plugin_tree_selection_changed_cb (dlg->selection, dlg);
 }
 
+/* ============================================================ */
+#define ITER_EQ(a,b) (((a).stamp == (b).stamp) && \
+                      ((a).user_data == (b).user_data) && \
+                      ((a).user_data2 == (b).user_data2) && \
+                      ((a).user_data3 == (b).user_data3))
+
+static gboolean
+gtk_tree_model_iter_prev (GtkTreeModel *tree_model, GtkTreeIter  *iter)
+{
+	int rc;
+	GtkTreeIter cur, prev;
+
+	rc = gtk_tree_model_get_iter_first (tree_model, &cur);
+	if (ITER_EQ (cur, *iter)) return 0;
+	while (rc)
+	{
+		prev = cur;
+		rc = gtk_tree_model_iter_next (tree_model, &cur);
+		if (0 == rc) break;
+		if (ITER_EQ (cur, *iter))
+		{
+			*iter = prev;
+			return 1;
+		}
+	}
+	return 0;
+}
+
+static void
+edit_plugin_set_selection (PluginEditorDialog *dlg, int offset)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean have_sel;
+	int rc;
+
+	have_sel = gtk_tree_selection_get_selected (dlg->selection, &model, &iter);
+	if (!have_sel) return;
+
+	if (0 < offset)
+	{
+		while (offset) 
+		{
+			rc = gtk_tree_model_iter_next (model, &iter);
+			if (0 == rc) return;
+			offset --;
+		}
+		gtk_tree_selection_select_iter (dlg->selection, &iter);
+	}
+	else
+	{
+		while (offset) 
+		{
+			rc = gtk_tree_model_iter_prev (model, &iter);
+			if (0 == rc) return;
+			offset ++;
+		}
+		gtk_tree_selection_select_iter (dlg->selection, &iter);
+	}
+}
+
+/* ============================================================ */
+/* Swap current selection with menu item at offset */
+
+static void 
+edit_plugin_move_menu_item (PluginEditorDialog *dlg, int offset)
+{
+	int row, rowb;
+	GnomeUIInfo *sysmenus, itema, itemb;
+
+	if (FALSE == dlg->have_selection) return;
+
+	/* Get selected item */
+	row = edit_plugin_get_index_of_selected_item (dlg);
+	if (-1 == row) return;
+
+	/* DO NOT move the end-of-array marker */
+	sysmenus = (GnomeUIInfo *) dlg->menus->data;
+	if (GNOME_APP_UI_ENDOFINFO == sysmenus[row].type) return;
+
+	rowb = row + offset;
+	if ((0 > rowb) || (rowb >= dlg->menus->len)) return;
+
+	itema = g_array_index (dlg->menus, GnomeUIInfo, row);
+	itemb = g_array_index (dlg->menus, GnomeUIInfo, rowb);
+	
+	g_array_index (dlg->menus, GnomeUIInfo, row) = itemb;
+	g_array_index (dlg->menus, GnomeUIInfo, rowb) = itema;
+
+	/* Redraw the tree */
+	dlg->have_selection = FALSE;
+	edit_plugin_redraw_tree (dlg);
+	edit_plugin_set_selection (dlg, offset);
+}
+
+static void 
+edit_plugin_up_button_cb (GtkWidget * w, gpointer data)
+{
+	edit_plugin_move_menu_item (data, -1);
+}
+
+static void 
+edit_plugin_down_button_cb (GtkWidget * w, gpointer data)
+{
+	edit_plugin_move_menu_item (data, 1);
+}
+
+/* ============================================================ */
+
+static void 
+edit_plugin_left_button_cb (GtkWidget * w, gpointer data)
+{
+	printf ("left button clicked\n");
+}
+
+/* ============================================================ */
+
+static void 
+edit_plugin_right_button_cb (GtkWidget * w, gpointer data)
+{
+	printf ("right button clicked\n");
+}
+
 /* ============================================================ */
 /* Create a new copy of the edit dialog; intialize all widgets, etc. */
 
@@ -565,6 +688,21 @@ edit_plugin_dialog_new (void)
 	glade_xml_signal_connect_data (gtxml, "on_plugin_tooltip_changed",
 		GTK_SIGNAL_FUNC (edit_plugin_changed_cb), dlg);
 	  
+	/* ------------------------------------------------------ */
+	/* Menu order change buttons */
+	
+	glade_xml_signal_connect_data (gtxml, "on_up_button_clicked",
+		GTK_SIGNAL_FUNC (edit_plugin_up_button_cb), dlg);
+	  
+	glade_xml_signal_connect_data (gtxml, "on_down_button_clicked",
+		GTK_SIGNAL_FUNC (edit_plugin_down_button_cb), dlg);
+	  
+	glade_xml_signal_connect_data (gtxml, "on_left_button_clicked",
+		GTK_SIGNAL_FUNC (edit_plugin_left_button_cb), dlg);
+	  
+	glade_xml_signal_connect_data (gtxml, "on_right_button_clicked",
+		GTK_SIGNAL_FUNC (edit_plugin_right_button_cb), dlg);
+	  
 	/* ------------------------------------------------------ */
 	/* Set up the Treeview Widget */
 	e = glade_xml_get_widget (gtxml, "editor treeview");
