@@ -21,13 +21,14 @@
 
 #include <glade/glade.h>
 #include <gnome.h>
-#include <libgnome/gnome-help.h>
+#include <qof.h>
 #include <string.h>
 
 #include "app.h"
 #include "ctree.h"
 #include "ctree-gnome2.h"
 #include "cur-proj.h"
+#include "dialog.h"
 #include "gtt.h"
 #include "prefs.h"
 #include "timer.h"
@@ -78,6 +79,9 @@ char *config_logfile_stop = NULL;
 int config_logfile_use = 0;
 int config_logfile_min_secs = 0;
 
+int config_daystart_offset = 0;
+int config_weekstart_offset = 0;
+
 char * config_data_url = NULL;
 
 typedef struct _PrefsDialog 
@@ -134,30 +138,123 @@ typedef struct _PrefsDialog
 	GtkCheckButton *show_tb_exit;
 
 	GtkEntry       *idle_secs;
+	GtkEntry       *daystart_secs;
+	GtkOptionMenu  *daystart_menu;
+	GtkOptionMenu  *weekstart_menu;
 } PrefsDialog;
 
 
 /* ============================================================== */
+/** Get and set the nth menu item */
 
-#define ENTRY_TO_CHAR(a, b) { 			\
-	const char *s = gtk_entry_get_text(a); 	\
-	if (s[0]) {				\
-		if (b) g_free(b); 		\
-		b = g_strdup(s); 		\
-	} else { 				\
-		if (b) g_free(b); 		\
-		b = NULL; 			\
-	} 					\
+static int
+get_optionmenu_item (GtkOptionMenu *opt_menu)
+{
+	GList *node;
+
+	/* Hunt for the selected item */
+	GtkWidget *menu = gtk_option_menu_get_menu (opt_menu);
+	int i=0;
+	GtkWidget *w = gtk_menu_get_active (GTK_MENU(menu));
+	for (node=GTK_MENU_SHELL(menu)->children; node; node=node->next)
+	{
+		if (w == node->data)
+		{
+			return i;
+		}
+		i++;
+	}
+	return 0;  /* can't possible reach this ... */
 }
 
-#define SHOW_CHECK(TOK) {					\
-	int state = GTK_TOGGLE_BUTTON(odlg->show_##TOK)->active;\
-	if (config_show_##TOK != state) {			\
-		change = 1;					\
-		config_show_##TOK = state;			\
-	}							\
+static void 
+set_optionmenu_item (GtkOptionMenu *opt_menu, int item)
+{
+	int i;
+	GList *node;
+	GtkWidget *w;
+	GtkMenuShell *menu;
+
+	/* Set the correct menu item based on current values */
+	w = gtk_option_menu_get_menu (opt_menu);
+	menu = GTK_MENU_SHELL(w);
+	i = 0;
+	for (node = menu->children; node; node=node->next)
+	{
+		if (item == i)
+		{
+			gtk_menu_shell_select_item (menu, node->data);
+			gtk_menu_shell_activate_item (menu, node->data, 1);
+			break;
+		}
+		i++;
+	}
 }
 
+/* ============================================================== */
+/** parse an HH:MM:SS string for the time returning seconds 
+ * XXX should probably use getdate or fdate or something like that 
+ */
+
+static int
+scan_time_string (const char *str)
+{
+	int hours=0, minutes=0, seconds = 0;
+	char buff[24];
+	strncpy (buff, str, 24);
+	buff[23]=0;
+	char * p = strchr (buff, ':');
+	if (p) *p = 0;
+	hours = atoi (buff);
+	if (p)
+	{
+		char *m = ++p;
+		p = strchr (m, ':');
+		if (p) *p = 0;
+		minutes = atoi (m);
+		if (p)
+		{
+			seconds = atoi(++p);
+		}
+	}
+	seconds %= 60;
+	minutes %= 60;
+	hours %= 24;
+
+	int totalsecs = hours*3600 + minutes*60 + seconds;
+	if (12*3600 < totalsecs) totalsecs -= 24*3600;
+	return totalsecs;
+}
+
+/* ============================================================== */
+
+#define ENTRY_TO_CHAR(a, b) {                               \
+   const char *s = gtk_entry_get_text(a);                   \
+   if (s[0]) {                                              \
+      if (b) g_free(b);                                     \
+      b = g_strdup(s);                                      \
+   } else {                                                 \
+      if (b) g_free(b);                                     \
+      b = NULL;                                             \
+   }                                                        \
+}
+
+#define SHOW_CHECK(TOK) {                                   \
+   int state = GTK_TOGGLE_BUTTON(odlg->show_##TOK)->active; \
+   if (config_show_##TOK != state) {                        \
+      change = 1;                                           \
+      config_show_##TOK = state;                            \
+   }                                                        \
+}
+
+#define SET_VAL(to,from) {                                  \
+   if (to != from) {                                        \
+      change = 1;                                           \
+      to = from;                                            \
+   }                                                        \
+}
+
+
 static void 
 prefs_set(GnomePropertyBox * pb, gint page, PrefsDialog *odlg)
 {
@@ -275,7 +372,23 @@ prefs_set(GnomePropertyBox * pb, gint page, PrefsDialog *odlg)
 
 	if (5 == page)
 	{
+		int change = 0;
 		config_idle_timeout = atoi(gtk_entry_get_text(GTK_ENTRY(odlg->idle_secs)));
+
+		/* Hunt for the hour-of night on which to start */
+		const char * buff = gtk_entry_get_text (odlg->daystart_secs);
+		int off = scan_time_string (buff);
+		SET_VAL (config_daystart_offset,off);
+
+		int day = get_optionmenu_item (odlg->weekstart_menu);
+		SET_VAL (config_weekstart_offset, day);
+
+		if (change)
+		{
+			/* Need to recompute everything, including the bining */
+			gtt_project_list_compute_secs();
+			ctree_refresh (global_ptw);
+		}
 	}
 
 	/* Also save them the to file at this point */
@@ -381,30 +494,73 @@ options_dialog_set(PrefsDialog *odlg)
 	g_snprintf(s, sizeof (s), "%d", config_idle_timeout);
 	gtk_entry_set_text(GTK_ENTRY(odlg->idle_secs), s);
 
+	/* Set the correct menu item based on current values */
+	int hour;
+	if (0<config_daystart_offset)
+	{
+		hour = (config_daystart_offset +1800)/3600;
+	}
+	else
+	{
+		hour = (config_daystart_offset -1800)/3600;
+	}
+	if (-3 > hour) hour = -3; /* menu runs from 9pm */
+	if (6 < hour) hour = 6;   /* menu runs till 6am */
+	hour += 3;  /* menu starts at 9PM */
+	set_optionmenu_item (odlg->daystart_menu, hour);
+
+	/* Print the daystart offset as a string in 24 hour time */
+	int secs = config_daystart_offset;
+	if (0 > secs) secs += 24*3600;
+	char buff[24];
+	qof_print_hours_elapsed_buff (buff, 24, secs, config_show_secs);   
+	gtk_entry_set_text (odlg->daystart_secs, buff);
+
+	/* Set the correct menu item based on current values */
+	int day = config_weekstart_offset;
+	set_optionmenu_item (odlg->weekstart_menu, day);
+
 	/* set to unmodified as it reflects the current state of the app */
 	gnome_property_box_set_modified(GNOME_PROPERTY_BOX(odlg->dlg), FALSE);
 }
 
 /* ============================================================== */
 
-#define GETWID(strname) 						\
-({									\
-	GtkWidget *e;							\
-	e = glade_xml_get_widget (gtxml, strname);			\
-	gtk_signal_connect_object(GTK_OBJECT(e), "changed",		\
-			  GTK_SIGNAL_FUNC(gnome_property_box_changed), 	\
-			  GTK_OBJECT(dlg->dlg));			\
-	e;								\
+static void 
+daystart_menu_changed (gpointer data, GtkOptionMenu *w)
+{
+	PrefsDialog *dlg = data;
+
+	int hour = get_optionmenu_item (dlg->daystart_menu);
+	hour += -3;  /* menu starts at 9PM */
+
+	int secs = hour * 3600;
+	if (0 > secs) secs += 24*3600;
+	char buff[24];
+	qof_print_hours_elapsed_buff (buff, 24, secs, config_show_secs);   
+	gtk_entry_set_text (dlg->daystart_secs, buff);
+}
+
+/* ============================================================== */
+
+#define GETWID(strname)                                            \
+({                                                                 \
+	GtkWidget *e;                                                   \
+	e = glade_xml_get_widget (gtxml, strname);                      \
+	gtk_signal_connect_object(GTK_OBJECT(e), "changed",             \
+	                  GTK_SIGNAL_FUNC(gnome_property_box_changed),  \
+	                  GTK_OBJECT(dlg->dlg));                        \
+	e;                                                              \
 })
 
-#define GETCHWID(strname) 						\
-({									\
-	GtkWidget *e;							\
-	e = glade_xml_get_widget (gtxml, strname);			\
-	gtk_signal_connect_object(GTK_OBJECT(e), "toggled",		\
-			  GTK_SIGNAL_FUNC(gnome_property_box_changed), 	\
-			  GTK_OBJECT(dlg->dlg));			\
-	e;								\
+#define GETCHWID(strname)                                          \
+({                                                                 \
+	GtkWidget *e;                                                   \
+	e = glade_xml_get_widget (gtxml, strname);                      \
+	gtk_signal_connect_object(GTK_OBJECT(e), "toggled",             \
+	                  GTK_SIGNAL_FUNC(gnome_property_box_changed),  \
+	                  GTK_OBJECT(dlg->dlg));                        \
+	e;                                                              \
 })
 
 static void 
@@ -539,16 +695,34 @@ misc_options(PrefsDialog *dlg)
 
 	w = GETWID ("idle secs");
 	dlg->idle_secs = GTK_ENTRY(w);
+
+	w = GETWID ("daystart entry");
+	dlg->daystart_secs = GTK_ENTRY(w);
+
+	w = GETWID ("daystart optionmenu");
+	dlg->daystart_menu = GTK_OPTION_MENU(w);
+
+	gtk_signal_connect_object(GTK_OBJECT(w), "changed",
+	                  GTK_SIGNAL_FUNC(daystart_menu_changed),
+	                  dlg);
+
+	w = GETWID ("weekstart optionmenu");
+	dlg->weekstart_menu = GTK_OPTION_MENU(w);
 }
 
 /* ============================================================== */
 
+static void 
+help_cb (GnomePropertyBox *propertybox, gint page_num, gpointer data)
+{
+	gtt_help_popup (propertybox, data);
+}
+
 static PrefsDialog *
 prefs_dialog_new (void)
 {
 	PrefsDialog *dlg;
 	GladeXML *gtxml;
-	// static GnomeHelpMenuEntry help_entry = { NULL, "preferences.html#PREF" };
 
 	dlg = g_malloc(sizeof(PrefsDialog));
 
@@ -557,12 +731,9 @@ prefs_dialog_new (void)
 
 	dlg->dlg = GNOME_PROPERTY_BOX (glade_xml_get_widget (gtxml,  "Global Preferences"));
 
-#if GNOME_20_HELP_IS_DIFFERENT
-	help_entry.name = gnome_app_id;
 	gtk_signal_connect(GTK_OBJECT(dlg->dlg), "help",
-			   GTK_SIGNAL_FUNC(gnome_help_pbox_display),
-			   &help_entry);
-#endif
+			   GTK_SIGNAL_FUNC(help_cb),
+			   "gnotime.xml#preferences");
 
 	gtk_signal_connect(GTK_OBJECT(dlg->dlg), "apply",
 			   GTK_SIGNAL_FUNC(prefs_set), dlg);
