@@ -19,30 +19,41 @@
 
 #include "config.h"
 
-#include <errno.h>
-#include <gnome.h>    /* only needed for definition of _() */
-/* #include <libintl.h> conflicts with <libgnome/gnome-i18n.h> on some systems */
+#include <glib.h>
+/* #include <gnome.h>  */  /* only needed for definition of _() */
+#include <libintl.h> /*conflicts with <libgnome/gnome-i18n.h> on some systems */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <qof.h>
+#include <qof/qofid-p.h>
+
 #include "err-throw.h"
 #include "log.h"
+#include "prefs.h"  /* XXX tmp hack for config_* */
 #include "proj.h"
 #include "proj_p.h"
+#include "query.h"  /* temp hack for query */
 
+#define _(X) gettext(X)
+
+/* XXX replace this notifier by GObject signals someday */
 typedef struct notif_s 
 {
 	GttProjectChanged func;
 	gpointer user_data;
 } Notifier;
 
-// hack alert -- plist should be made static/private to this file
+// hack alert -- plist should be made to belong to a book
 GList * plist = NULL;
 
+QofBook *global_book = NULL;
+
 static void proj_refresh_time (GttProject *proj);
 static void proj_modified (GttProject *proj);
 static int task_suspend (GttTask *tsk);
+static void gtt_interval_unhook (GttInterval * ivl);
 
 /* ============================================================= */
 
@@ -98,6 +109,7 @@ gtt_project_new(void)
 	proj->id = next_free_id;
 	next_free_id ++;
 
+	qof_instance_init (&proj->inst, GTT_PROJECT_ID, global_book);
 	return proj;
 }
 
@@ -239,6 +251,18 @@ gtt_project_destroy(GttProject *proj)
 	g_free(proj);
 }
 
+void 
+gtt_project_set_guid (GttProject *prj, const GUID *guid)
+{
+	qof_entity_set_guid (&prj->inst.entity, guid);
+}
+
+ 
+const GUID *
+gtt_project_get_guid (GttProject *prj)
+{
+	return qof_entity_get_guid (&prj->inst.entity);
+}
 
 /* ========================================================= */
 /* set/get project titles and descriptions */
@@ -611,12 +635,18 @@ get_midnight (time_t last)
 		last = time(0);
 	}
 
+	/* If config_daystart_offset == 3*3600 then the new day
+	 * will start at 3AM, for example.  */
+	last -= config_daystart_offset;
+
 	memcpy (&lt, localtime (&last), sizeof (struct tm));
 	lt.tm_sec = 0;
 	lt.tm_min = 0;
 	lt.tm_hour = 0;
 	midnight = mktime (&lt);
 
+	midnight += config_daystart_offset;
+
 	return midnight;
 }
 
@@ -638,6 +668,10 @@ get_sunday (time_t last)
 	lt.tm_mday -= lt.tm_wday;
 	sunday = mktime (&lt);
 
+	/* If config_weekstart_offset == 1 then a new week starts
+	 * on monday, not sunday. */
+	sunday += 24*3600* config_weekstart_offset + config_daystart_offset;
+
 	return sunday;
 }
 
@@ -659,6 +693,7 @@ get_month (time_t last)
 	lt.tm_mday = 1;
 	first = mktime (&lt);
 
+	first += config_daystart_offset;
 	return first;
 }
 
@@ -680,6 +715,7 @@ get_newyear (time_t last)
 	lt.tm_mday -= lt.tm_yday;
 	newyear = mktime (&lt);
 
+	newyear += config_daystart_offset;
 	return newyear;
 }
 
@@ -690,10 +726,10 @@ gtt_project_compat_set_secs (GttProject *proj, int sever, int sday, time_t last)
 	GttTask *tsk;
 	GttInterval *ivl;
 
-	/* get the midnight of the last update */
+	/* Get the midnight of the last update */
 	midnight = get_midnight (last);
 
-	/* old GTT data will just be one big interval 
+	/* Old GTT data will just be one big interval 
 	 * lumped under one task */
 	tsk = gtt_task_new ();
 	gtt_task_set_memo (tsk, _("Old GTT Tasks"));
@@ -1221,6 +1257,7 @@ gtt_project_list_total (void)
 	return total;
 }
 
+/* =========================================================== */
 /* =========================================================== */
 /* Recomputed cached data.  Scrub it while we're at it. */
 /* Scrub algorithm as follows (as documented in header files):
@@ -1332,7 +1369,7 @@ scrub_intervals (GttTask *tsk, GttInterval *handle)
 			{
 				GttInterval *nivl = node->next->data;
 
-				/* merge only if the intervals are in the same day */
+				/* Merge only if the intervals are in the same day */
 				if (get_midnight (ivl->start) == get_midnight (nivl->stop))
 				{
 					gap_down = ivl->start - nivl->stop;
@@ -1545,8 +1582,8 @@ gtt_project_list_compute_secs (void)
 
 void
 gtt_project_add_notifier (GttProject *prj, 
-                          GttProjectChanged cb, 
-                          gpointer user_stuff)
+                       GttProjectChanged cb, 
+                       gpointer user_stuff)
 {
 	Notifier * ntf;
 
@@ -1560,8 +1597,8 @@ gtt_project_add_notifier (GttProject *prj,
 
 void
 gtt_project_remove_notifier (GttProject *prj, 
-                          GttProjectChanged cb, 
-                          gpointer user_stuff)
+                       GttProjectChanged cb, 
+                       gpointer user_stuff)
 {
 	Notifier * ntf;
 	GList *node;
@@ -1850,6 +1887,82 @@ gtt_project_timer_stop (GttProject *proj)
 
 /* =========================================================== */
 
+static void
+prj_obj_foreach_helper (GList *prjlist, QofEntityForeachCB cb, gpointer data)
+{
+	GList *node;
+
+	for (node=prjlist; node; node=node->next)
+	{
+		GttProject *prj = node->data;
+		void (*ncb) (GttProject *, void *) = (void (*) (GttProject *, void *)) cb;
+		ncb (prj, data);
+		prj_obj_foreach_helper (prj->sub_projects, cb, data);
+	}
+}
+
+void
+gtt_project_obj_foreach (QofCollection *coll, QofEntityForeachCB cb, gpointer data)
+{
+	prj_obj_foreach_helper (plist, cb, data);
+}
+
+static int
+prj_obj_order (GttProject *a, GttProject *b)
+{
+	return 0;
+}
+
+static QofObject prj_object_def =
+{
+interface_version: QOF_OBJECT_VERSION,
+e_type:            GTT_PROJECT_ID,
+type_label:        GTT_PROJECT_ID,
+new:               NULL,
+book_begin:        NULL,
+book_end:          NULL,
+is_dirty:          NULL,
+mark_clean:        NULL,
+foreach:           gtt_project_obj_foreach,
+printable:         NULL,
+};
+
+/* =========================================================== */
+/* bogus wrappers */
+static Timespec 
+prj_obj_get_earliest (GttProject *prj)
+{
+	Timespec ts = {0,0};
+	ts.tv_sec = gtt_project_get_earliest_start (prj, TRUE);
+	return ts;
+}
+
+static Timespec
+prj_obj_get_latest (GttProject *prj)
+{
+	Timespec ts = {0,0};
+	ts.tv_sec = gtt_project_get_latest_stop (prj, TRUE);
+	return ts;
+}
+
+gboolean 
+gtt_project_obj_register (void)
+{
+	global_book = qof_book_new();
+
+/* Associate an ASCII name to each getter, as well as the return type */
+static QofParam params[] = {
+		{ GTT_PROJECT_EARLIEST, QOF_TYPE_DATE, (QofAccessFunc)prj_obj_get_earliest, NULL},
+		{ GTT_PROJECT_LATEST, QOF_TYPE_DATE, (QofAccessFunc)prj_obj_get_latest, NULL},
+		{ NULL },
+	};
+
+	qof_class_register (GTT_PROJECT_ID, (QofSortFunc)prj_obj_order, params);
+	return qof_object_register (&prj_object_def);
+}
+
+/* =========================================================== */
+
 GttTask *
 gtt_task_new (void)
 {
@@ -1864,11 +1977,13 @@ gtt_task_new (void)
 	task->billstatus = GTT_BILL,
 	task->bill_unit = 900;
 	task->interval_list = NULL;
+
+	qof_instance_init (&task->inst, GTT_TASK_ID, global_book);
 	return task;
 }
 
 GttTask *
-gtt_task_dup (GttTask *old)
+gtt_task_copy (GttTask *old)
 {
 	GttTask *task;
 
@@ -1886,6 +2001,7 @@ gtt_task_dup (GttTask *old)
 	task->bill_unit = old->bill_unit;
 	task->interval_list = NULL;
 
+	qof_instance_init (&task->inst, GTT_TASK_ID, global_book);
 	return task;
 }
 
@@ -1996,11 +2112,9 @@ gtt_task_new_insert (GttTask *old)
 
 	if (!old) return NULL;
 
-	task = g_new0(GttTask, 1);
-	task->memo = g_strdup (_("New Diary Entry"));
-	task->notes = g_strdup ("");
+	task = gtt_task_new();
 
-	/* inherit the properties ... important for user */
+	/* Inherit the properties ... important for user */
 	task->billable = old->billable;
 	task->billrate = old->billrate;
 	task->billstatus = old->billstatus;
@@ -2023,10 +2137,23 @@ gtt_task_new_insert (GttTask *old)
 	return task;
 }
 
+void 
+gtt_task_set_guid (GttTask *tsk, const GUID *guid)
+{
+	qof_entity_set_guid (&tsk->inst.entity, guid);
+}
+
+const GUID *
+gtt_task_get_guid (GttTask *tsk)
+{
+	return qof_entity_get_guid (&tsk->inst.entity);
+}
+
 void
 gtt_task_add_interval (GttTask *tsk, GttInterval *ival)
 {
 	if (!tsk || !ival) return;
+	gtt_interval_unhook (ival);
 	tsk->interval_list = g_list_prepend (tsk->interval_list, ival);
 	ival->parent = tsk;
 	proj_refresh_time (tsk->parent);
@@ -2036,6 +2163,7 @@ void
 gtt_task_append_interval (GttTask *tsk, GttInterval *ival)
 {
 	if (!tsk || !ival) return;
+	gtt_interval_unhook (ival);
 	tsk->interval_list = g_list_append (tsk->interval_list, ival);
 	ival->parent = tsk;
 	proj_refresh_time (tsk->parent);
@@ -2159,6 +2287,16 @@ gtt_task_is_first_task (GttTask *tsk)
 	return FALSE;
 }
 
+gboolean
+gtt_task_is_last_task (GttTask *tsk)
+{
+	if (!tsk || !tsk->parent || !tsk->parent->task_list) return TRUE;
+	
+	GList *last = g_list_last (tsk->parent->task_list);
+	if ((GttTask *) last->data == tsk) return TRUE;
+	return FALSE;
+}
+
 GttProject * 
 gtt_task_get_parent (GttTask *tsk)
 {
@@ -2212,6 +2350,38 @@ gtt_task_get_secs_ever (GttTask *tsk)
 	return total;
 }
 
+time_t
+gtt_task_get_secs_earliest (GttTask *tsk)
+{
+	GList *node;
+	if (NULL == tsk->interval_list) return 0;
+
+	time_t earliest = INT_MAX;
+
+	for (node=tsk->interval_list; node; node=node->next)
+	{
+		GttInterval * ivl = node->data;
+		if (ivl->start < earliest) earliest = ivl->start;
+	}
+	return earliest;
+}
+
+time_t
+gtt_task_get_secs_latest (GttTask *tsk)
+{
+	GList *node;
+	if (NULL == tsk->interval_list) return 0;
+
+	time_t latest = INT_MIN;
+
+	for (node=tsk->interval_list; node; node=node->next)
+	{
+		GttInterval * ivl = node->data;
+		if (ivl->stop > latest) latest = ivl->stop;
+	}
+	return latest;
+}
+
 /* =========================================================== */
 
 GttInterval *
@@ -2231,17 +2401,22 @@ void
 gtt_interval_destroy (GttInterval * ivl)
 {
 	if (!ivl) return;
-	if (ivl->parent)
-	{
-		/* Unhook myself from the chain */
-		ivl->parent->interval_list = 
-			g_list_remove (ivl->parent->interval_list, ivl);
-		proj_refresh_time (ivl->parent->parent);
-	}
-	ivl->parent = NULL;
+	gtt_interval_unhook (ivl);
 	g_free (ivl);
 }
 
+static void 
+gtt_interval_unhook (GttInterval * ivl)
+{
+	if (NULL == ivl->parent) return;
+
+	/* Unhook myself from the chain */
+	ivl->parent->interval_list = 
+		g_list_remove (ivl->parent->interval_list, ivl);
+	proj_refresh_time (ivl->parent->parent);
+	ivl->parent = NULL;
+}
+
 void
 gtt_interval_set_start (GttInterval *ivl, time_t st)
 {
@@ -2734,9 +2909,10 @@ cmp_status(const void *aa, const void *bb)
 }
 
 #define DO_SORT(CMP_FUNC)                                   \
-   gboolean is_top = FALSE;                                 \
-   GttProject *parent;                                      \
-   if (prjs == plist) is_top = TRUE;                        \
+	gboolean is_top = FALSE;                                 \
+	GttProject *parent;                                      \
+	if (!prjs) return NULL;                                  \
+	if (prjs == plist) is_top = TRUE;                        \
 	parent = ((GttProject *) (prjs->data))->parent;          \
 	prjs = project_list_sort(prjs, CMP_FUNC);                \
 	if (is_top) plist = prjs;                                \
