@@ -21,6 +21,7 @@
 #define __GTT_PROJ_H__
 
 #include <glib.h>
+#include <qof.h>
 
 /* The data structures for GnoTime are written in a quasi-object-oriented
  * style.  All data is encapsulated in opaque structs, and can be accessed
@@ -84,7 +85,16 @@ typedef enum
 	GTT_COMPLETED
 } GttProjectStatus;
 		
+/* -------------------------------------------------------- */
+/* Query related things -- under construction */
 
+#define GTT_PROJECT_ID "GttProjectId"
+#define GTT_TASK_ID    "GttTaskId"
+
+#define GTT_PROJECT_EARLIEST "GttProjectEarliet"
+#define GTT_PROJECT_LATEST   "GttProjectLatest"
+
+/* -------------------------------------------------------- */
 /* The three basic structures */
 typedef struct gtt_project_s GttProject;
 typedef struct gtt_task_s GttTask;
@@ -94,6 +104,10 @@ typedef void (*GttProjectChanged) (GttProject *, gpointer);
 typedef int (*GttProjectCB) (GttProject *, gpointer);
 typedef int (*GttIntervalCB) (GttInterval *, gpointer);
 
+/* -------------------------------------------------------- */
+/* system init */
+gboolean gtt_project_obj_register (void);
+
 /* -------------------------------------------------------- */
 /* project data */
 
@@ -116,6 +130,8 @@ GttProject *	gtt_project_dup(GttProject *);
  */
 void 		gtt_project_remove(GttProject *p);
 
+const GUID * gtt_project_get_guid (GttProject *);
+
 void 		gtt_project_set_title(GttProject *, const char *);
 void 		gtt_project_set_desc(GttProject *, const char *);
 void 		gtt_project_set_notes(GttProject *, const char *);
@@ -469,15 +485,17 @@ int      gtt_project_list_total (void);
  * 'diary entries'.
  */
 
-/* The gtt_task_dup() routine makes a copy of the indicated task.
- *    it copies the memo, notes, adn billing info, but not the 
+/* The gtt_task_copy() routine makes a copy of the indicated task.
+ *    it copies the memo, notes, and billing info, but not the 
  *    intervals, nor the parent.
  */
 
 GttTask *	gtt_task_new (void);
-GttTask *	gtt_task_dup (GttTask *);
+GttTask *	gtt_task_copy (GttTask *);
 void 		gtt_task_destroy (GttTask *);
 
+const GUID * gtt_task_get_guid (GttTask *);
+
 void		gtt_task_set_memo (GttTask *, const char *);
 const char *	gtt_task_get_memo (GttTask *);
 void		gtt_task_set_notes (GttTask *, const char *);
@@ -513,12 +531,16 @@ int		gtt_task_get_bill_unit (GttTask *);
  *
  * The gtt_task_is_first_task() routine returns True if this task
  *    is the leading task of the project.
+ *    
+ * The gtt_task_is_last_task() routine returns True if this task
+ *    is the last task of the project.
  */
 void         gtt_task_remove (GttTask *);
 GttTask *    gtt_task_new_insert (GttTask *);
 void         gtt_task_insert (GttTask *where, GttTask *insertee);
 void         gtt_task_merge_up (GttTask *);
 gboolean     gtt_task_is_first_task (GttTask *);
+gboolean     gtt_task_is_last_task (GttTask *);
 GttProject * gtt_task_get_parent (GttTask *);
 
 
@@ -528,8 +550,14 @@ void		gtt_task_append_interval (GttTask *, GttInterval *);
 
 /* gtt_task_get_secs_ever() adds up and returns the total number of 
  * seconds in the intervals in this task. */
-int 		gtt_task_get_secs_ever (GttTask *tsk);
+int     gtt_task_get_secs_ever (GttTask *tsk);
 
+/* Get the earliest and the latest timestamps to occur in any
+ * intervals associated with this task.  Return 0 if there
+ * are  o intervals on this task.
+ */
+time_t   gtt_task_get_secs_earliest (GttTask *tsk);
+time_t   gtt_task_get_secs_latest (GttTask *tsk);
 
 /* intervals */
 /* The gtt_interval_set_start() and gtt_interval_set_stop() set the
